{****************************************}
{* Directory backup program for FreeDOS *}
{****************************************}

program backup;

uses
  Dos, SysUtils, IniFiles;

type
  TOptions = array of string;

const
  BatchFile = 'backitup.bat';

var
  Err, N, P: Integer;
  ConfigPath, SourcePath, BackupPath, BackupName, BackupDir, Mess, SN: string;
  cmd, params: string;
  Found: Boolean;
  Options: TOptions;

{ Read program parameters from command line }

procedure GetParams(var Source, Dest, Backup: string; var Options: TOptions; var Mess: string);

var
  I, J: Integer;
  S: string;

begin
  SetLength(Options, 0); Mess := '';
  if (ParamCount < 2) or (ParamCount > 5) then
    Mess := 'Invalid number of command line parameters'
  else if not DirectoryExists(ParamStr(1)) then
    Mess := 'Invalid source directory'
  else if not DirectoryExists(ParamStr(2)) then
    Mess := 'Invalid backup directory'
  else begin
    Source := ParamStr(1); Dest := ParamStr(2); Backup := 'bak';
    if ParamCount > 2 then begin
      I := 3;
      if LeftStr(ParamStr(I), 1) <> '/' then begin
        Backup := ParamStr(I); Inc(I);
        if Length(Backup) > 6 then
          Mess := 'Backup name to long';
      end;
      if Mess = '' then begin
        // Program options
        for J := I to ParamCount do begin
          S := UpperCase(ParamStr(J));
          if (S <> '/H') and (S <> '/S') and (S <> '/E') then begin
            if Mess = '' then
              Mess := 'Invalid program option ' + S;
          end
          else begin
            SetLength(Options, Length(Options) + 1); Options[Length(Options) - 1] := S;
          end;
        end;
        if Mess = '' then begin
          if Length(Options) > 2 then
            Mess := 'Invalid number of command line options'
          else begin
            if Length(Options) = 2 then begin
              if ((Options[0] <> '/S') and (Options[1] <> '/E')) or ((Options[0] <> '/E') and (Options[1] <> '/S')) then
                Mess := 'Incompatible options /S and /E'
              else if Options[0] = Options[1] then
                Delete(Options, 1, 1);
            end;
          end;
        end;
      end;
    end;
  end;
end;

{ Read program parameters from config file }

procedure ReadConfigFile(Config: string; var Source, Dest, Backup: string; var Options: TOptions; var Mess: string);

var
  I: Integer;
  S: string;
  ConfigFile: TINIFile;

begin
  SetLength(Options, 0); Mess := '';
  ConfigFile := TINIFile.Create(Config);
  //try
    Source := ConfigFile.ReadString('Source', 'Path', '');
    Dest := ConfigFile.ReadString('Destination', 'Path', '');
    if not DirectoryExists(Source) then
      Mess := 'Missing or invalid source directory'
    else if not DirectoryExists(Dest) then
      Mess := 'Missing or invalid backup directory'
    else begin
      Backup := ConfigFile.ReadString('Destination', 'Backup', '');
      if Backup = '' then
        Backup := 'bak';
      if Length(Backup) > 6 then
        Mess := 'Backup name to long'
      else begin
        I := 0;
        S := ConfigFile.ReadString('Options', 'HiddenFiles', '');
        if S = 'yes' then begin
          Inc(I); SetLength(Options, I); Options[I - 1] := '/H';
        end
        else if S <> 'no' then begin
          Mess := 'Invalid value for option HiddenFiles';
        end;
      end;
      if Mess = '' then begin
        S := ConfigFile.ReadString('Options', 'SubDirs', '');
        if (S = 'yes') or (S = 'all') then begin
          Inc(I); SetLength(Options, I);
          if S = 'yes' then
            Options[I - 1] := '/S'
           else
             Options[I - 1] := '/E';
        end
        else if S <> 'no' then begin
          Mess := 'Invalid value for option SubDirs';
        end;
      end;
    end;
  //except
    //Mess := 'Error when reading backup.ini file';
  //end;
  ConfigFile.Free;
end;

{ Create batch file backitup.bat }

procedure CreateBatchFile(BatchFile, SourcePath, BackupDir: string; var Options: TOptions);

var
  I: Integer;
  S: string;
  FOut: Text;

begin
  if FileExists(BatchFile) then
    DeleteFile(BatchFile);
  Assign(FOut, BatchFile); Rewrite(FOut);
  Writeln(FOut, '@echo off');
  Writeln(FOut, 'mkdir ', BackupDir);
  S := 'xcopy ' + SourcePath + ' ' + BackupDir;
  if Length(Options) >= 1 then begin
    for I := 0 to Length(Options) do
      S := S + ' ' + Options[I];
  end;
  S := S + ' /I /Q';
  Writeln(FOut, S);
  Close(FOut);
end;

{**************}
{ Main program }
{**************}

begin
  Writeln; Writeln('Directory backup for FreeDOS, v1.0, (c) allu April 2023 - June 2025.'); Writeln;
  if LowerCase(GetEnv('OS_NAME')) <> 'freedos' then begin
    // The program should not be used on an operating system other than FreeDOS!
    Writeln('Sorry! This program is intended for FreeDOS!');
    Halt(1);
  end;
  ConfigPath := '';
  if ParamCount = 0 then begin
    // If no command line parameter, use program parameters from default config file
    ConfigPath := GetEnv('DOSDIR') + '\backup.ini';
  end
  else if ParamCount = 1 then begin
    // If one single command line parameter, this is supposed to be the full path to the config file
    ConfigPath := ParamStr(1);
  end;
  if ConfigPath = '' then begin
    // Read program parameters from command line
    GetParams(SourcePath, BackupPath, BackupName, Options, Mess);
  end
  else begin
    // Read program parameters from config file
    if not FileExists(ConfigPath) then
      Mess := 'Configuration file ' + ConfigPath + ' not found'
    else begin
      ReadConfigFile(ConfigPath, SourcePath, BackupPath, BackupName, Options, Mess);
    end;
  end;
  if Mess = '' then begin
    // Program parameters are ok so far; remains to test if the destination is not a subdirectory of the source
    P := Pos(SourcePath, BackupPath);
    if P = 0 then begin
      // Ok. Determine actual backup directory suffix
      Found := False; N := 0;
      repeat
        Inc(N);
        SN := IntToStr(N);
        if N < 10 then
          SN := '0' + SN;
        BackupDir := BackupPath + '\' + BackupName + SN;
        if not DirectoryExists(BackupDir) then
          Found := True;
      until Found;
      if N < 100 then begin
        // Create batch file, containing the FreeDOS commands to copy the files
        CreateBatchFile(BatchFile, SourcePath, BackupDir, Options);
        Writeln('Please, run ' + BatchFile + ' to copy your files to ' + BackupDir + '...'); Writeln;
      end
      else
        Mess := 'Number of backups cannot exceed 99';
    end
    else
      Mess := 'Backup destination cannot be a subdirectory of the source';
  end;
  if Mess <> '' then
    Writeln('Backup failure: ', Mess, '!'); Writeln;
end.
